#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function
from __future__ import absolute_import
import argparse
import pkg_resources
import logging
import sys

import aadbook.config
from .contacts import Contacts

log = logging.getLogger(__name__)

CONFIG_FILE = '~/.aadbookrc'


def do_config_template(config, args):
    print(aadbook.config.TEMPLATE)


def do_authenticate(config, args):
    auth = config.auth

    auth.authenticate(ignore_refresh_token=True)
    print('Authentication succeeded')


def do_reload(config, args):
    contacts = Contacts(config)
    contacts.reload()


def do_query(config, args):
    contacts = Contacts(config)
    contacts.query(args.query)


def _build_parser():
    """
    Return a command-line arguments parser.
    """
    parser = argparse.ArgumentParser(description='Search you Azure AD contacts from mutt or the command-line.')
    parser.add_argument('-c', '--config', help='Specify alternative configuration file.', metavar="FILE")
    parser.add_argument('-v', '--verbose', dest="log_level", action='store_const',
                        const=logging.INFO, help='Be verbose about what is going on (stderr).')
    parser.add_argument('-V', '--version',
                        action='version',
                        version='%%(prog)s %s' % pkg_resources.get_distribution("aadbook").version,
                        help="Print version and exit")
    parser.add_argument('-d', '--debug', dest="log_level", action='store_const',
                        const=logging.DEBUG, help='Output debug info (stderr).')
    parser.set_defaults(config=CONFIG_FILE, log_level=logging.ERROR)

    subparsers = parser.add_subparsers()

    parser_config_template = subparsers.add_parser('config-template',
                                                   description='Prints a template for .aadbookrc to stdout')
    parser_config_template.set_defaults(func=do_config_template)

    parser_reload = subparsers.add_parser('authenticate',
                                          description='Azure AD authentication.')
    parser_reload.set_defaults(func=do_authenticate)

    parser_reload = subparsers.add_parser('reload',
                                          description='Force reload of the cache.')
    parser_reload.set_defaults(func=do_reload)

    parser_query = subparsers.add_parser('query',
                                         description='Search contacts using query (regex).')
    parser_query.add_argument('query', help='regex to search for.', metavar='QUERY')
    parser_query.set_defaults(func=do_query)

    return parser


def _main():
    parser = _build_parser()

    args = [arg.decode(aadbook.config.ENCODING) for arg in sys.argv[1:]]
    args = parser.parse_args(args)

    logging.basicConfig(level=args.log_level)

    try:
        if args.func == do_config_template:
            config = None
        else:
            config = aadbook.config.read_config(args.config)
        args.func(config, args)
    except aadbook.config.ConfigError as err:
        sys.exit(u'Configuration error: ' + unicode(err))


if __name__ == '__main__':
    _main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import itertools
import logging
import os
import time
import pickle
import re
import sys


import requests
from collections import namedtuple

log = logging.getLogger(__name__)

Contact = namedtuple('Contact', 'name mail'.split())

CACHE_FORMAT_VERSION = '1.0'
MAX_USERS_PER_PAGE = 999


class Cache(object):
    def __init__(self, config):
        self._config = config
        self.contacts = self._load_contacts()

    def _load_contacts(self):
        cache = None

        # if cache newer than cache_expiry_hours
        if (os.path.exists(self._config.cache_filename) and
            ((time.time() - os.path.getmtime(self._config.cache_filename)) <
             (int(self._config.cache_expiry_hours) * 60 * 60))):
            try:
                log.debug('Loading cache: ' + self._config.cache_filename)
                cache = pickle.load(open(self._config.cache_filename, 'rb'))
                if cache.get('aadbook_cache') != CACHE_FORMAT_VERSION:
                    log.info('Detected old cache format')
                    cache = None  # Old cache format
            except StandardError, err:
                log.info('Failed to read the cache file: %s', err)
                raise
        if cache:
            return cache.get('contacts')

    def update(self, contacts):
        self.contacts = contacts
        self.save()

    def save(self):
        """Pickle the addressbook and a timestamp

        """
        if self.contacts:  # never write a empty addressbook
            cache = {'contacts': self.contacts,
                     'aadbook_cache': CACHE_FORMAT_VERSION}
            pickle.dump(cache, open(self._config.cache_filename, 'wb'))


class Contacts(object):
    def __init__(self, config):
        self._config = config
        self._auth = config.auth
        self.cache = Cache(config)

    def reload(self):
        all_users = []
        for users in self._fetch_users():
            sys.stdout.write('.')
            sys.stdout.flush()
            all_users.extend(users)

        self.cache.update(all_users)

    def _fetch_users(self):
        session = requests.Session()
        session.headers.update({
            'Authorization': "Bearer " + self._auth.creds['accessToken']
        })
        response = session.get(self._users_url).json()
        [users, next] = self._parse_users_response(response)
        yield users
        while next:
            response = session.get(next).json()
            [users, next] = self._parse_users_response(response)
            yield users

    @property
    def _users_url(self):
        return ('https://graph.microsoft.com/v1.0/users?$top=%s'
                % MAX_USERS_PER_PAGE)

    def _parse_users_response(self, response):
        def parse_contact(account):
            return Contact(account['displayName'], account['mail'])

        if 'value' not in response:
            raise ContactsParseError("Failed to parse contacts\n%s" % response)

        users = map(parse_contact, response['value'])
        next = (response['@odata.nextLink']
                if '@odata.nextLink' in response else None)
        return [users, next]

    def query(self, query):
        if not self.cache.contacts:
            self.reload()

        matching_contacts = sorted(self.__query_contacts(query),
                                   key=lambda c: c.name)
        # mutt's query_command expects the first line to be a message,
        # which it discards.
        print "\n",
        for contact in matching_contacts:
            name = contact.name
            mail = contact.mail
            print (u'\t'.join((mail, name))).encode(self._config.encoding,
                                                    errors='replace')

    def __query_contacts(self, query):
        match = re.compile(query.replace(' ', '.*'), re.I).search
        for contact in self.cache.contacts:
            if contact.mail:
                if any(itertools.imap(match,
                                      [contact.name, contact.mail])):
                    yield contact


class ContactsParseError(StandardError):
    pass



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import logging
import json
from datetime import datetime
from dateutil.parser import parse

import adal

log = logging.getLogger(__name__)

AUTHORITY_HOST_URI = 'https://login.microsoftonline.com'
TENANT = 'common'
AUTHORITY_URI = AUTHORITY_HOST_URI + '/' + TENANT
RESOURCE_URI = 'https://graph.microsoft.com'
CLIENT_ID = '9f461068-2572-465b-9377-a5c284b326d1'


class Auth(object):
    def __init__(self, auth_file):
        self.context = adal.AuthenticationContext(AUTHORITY_URI,
                                                  api_version=None)
        self.auth_file = auth_file
        self.creds = self._read_creds()
        if self.creds and self.invalid:
            try:
                self.authenticate()
                self.creds = self._read_creds()
            except adal.adal_error.AdalError as err:
                log.error("Failed to refresh tokens: %s", unicode(err))
                self.creds = None

    def _read_creds(self):
        try:
            with open(self.auth_file, 'r') as f:
                return json.load(f)
        except IOError:
            log.debug("Failed to read auth_file %s" % self.auth_file)
            return None

    @property
    def invalid(self):
        if not self.creds:
            return True

        expires_on = parse(self.creds['expiresOn'])
        return expires_on < datetime.now()

    def authenticate(self, ignore_refresh_token=False):
        if not ignore_refresh_token and self.creds:
            refresh_token = self.creds['refreshToken']
            token = self.context.acquire_token_with_refresh_token(
                refresh_token,
                CLIENT_ID,
                RESOURCE_URI)
        else:
            code = self.context.acquire_user_code(RESOURCE_URI,
                                                  CLIENT_ID)
            print(code['message'])
            token = self.context.acquire_token_with_device_code(RESOURCE_URI,
                                                                code,
                                                                CLIENT_ID)
        self._save_creds(token)

    def _save_creds(self, data):
        with open(self.auth_file, 'w') as f:
            json.dump(data, f)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from __future__ import absolute_import

import ConfigParser
import locale
import logging
import os

from .auth import Auth
from .storage import Storage
from os.path import realpath, expanduser

log = logging.getLogger(__name__)

try:
    ENCODING = locale.getpreferredencoding() or 'UTF-8'
except LookupError:
    # Some OS X can give a strange encoding
    ENCODING = 'UTF-8'

TEMPLATE = '''\
# "#" or ";" at the start of a line makes it a comment.
[DEFAULT]
# The following are optional, defaults are shown

# This file is written by the Azure AD library, and should be kept secure,
# it's like a password to your AD contacts.
;auth_db_filename: ~/.aadbook_auth.json

;cache_filename: ~/.aadbook_cache
;cache_expiry_hours: 24
'''


def read_config(config_file):
    '''Reads the ~/.aadbookrc and any authentication data
    returns the configuration as a dictionary.

    '''
    config = Storage({  # Default values
        'cache_filename': '~/.aadbook_cache',
        'auth_db_filename': '~/.aadbook_auth.json',
        'cache_expiry_hours': '24'})
    config_file = os.path.expanduser(config_file)
    parser = _get_config(config_file)
    if parser:
        config.get_dict().update(dict(parser.items('DEFAULT', raw=True)))

    # Ensure paths are fully expanded
    config.cache_filename = realpath(expanduser(config.cache_filename))
    config.auth_db_filename = realpath(expanduser(config.auth_db_filename))

    config.auth = Auth(config.auth_db_filename)
    # config.creds = config.store.get()

    # What terminal encoding to use
    config.encoding = ENCODING

    log.debug(config)
    return config


def _get_config(config_file):
    '''find, read and parse configuraton.'''
    parser = ConfigParser.SafeConfigParser()
    if os.path.lexists(config_file):
        try:
            log.info('Reading config: %s', config_file)
            inp = open(config_file)
            parser.readfp(inp)
            return parser
        except (IOError, ConfigParser.ParsingError), err:
            raise ConfigError("Failed to read configuration %s\n%s" % (config_file, err))
    return None


class ConfigError(StandardError):
    pass



#!/usr/bin/env python

import os
from setuptools import find_packages
from setuptools import setup


def get_version():
    import subprocess
    from aadbook import __version__ as reference_tag

    proc = subprocess.Popen('git rev-list %s..HEAD --count' % (reference_tag,),
                            shell=True,
                            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    pending, _ = proc.communicate()
    if not pending:
        return reference_tag
    return "%s-r%s" % (reference_tag, pending)


VERSION = get_version()
NAME = 'aadbook'
PACKAGES = [NAME]
DESCRIPTION = 'AADBook -- Access your Azure AD contacts from the command line'
LICENSE = 'GPLv3'
readme = os.path.join(os.path.dirname(__file__), 'README.rst')
LONG_DESCRIPTION = open(readme).read()
INSTALL_REQUIRES = [
    'adal==1.0.2',
    'requests==2.20.0'
]

URL = 'https://github.com/iamFIREcracker/aadbook'
DOWNLOAD_URL = 'http://pypi.python.org/pypi/aadbook'

CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Environment :: Console',
    'Intended Audience :: End Users/Desktop',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.2',
    'License :: OSI Approved :: GNU General Public License (GPL)',
    'Topic :: Communications :: Email :: Address Book',
]
AUTHOR = 'Matteo Landi'
AUTHOR_EMAIL = 'matteo@matteolandi.net'
KEYWORDS = "contacts azure active directory ad".split(' ')

params = dict(
    name=NAME,
    version=VERSION,
    packages=find_packages(),
    entry_points={
        'console_scripts': ['aadbook = aadbook.aadbook:_main'],
    },
    install_requires=INSTALL_REQUIRES,

    # metadata for upload to PyPI
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    license=LICENSE,
    keywords=KEYWORDS,
    url=URL,
    download_url=DOWNLOAD_URL,
    classifiers=CLASSIFIERS,
)

setup(**params)
